#include "gldraw.hpp"

CGLDraw::CGLDraw(void)
{
}

CGLDraw::~CGLDraw(void)
{
}

// Load a VQ texture
int CGLDraw::LoadVqTxr(const char *fn, int img_size, GLuint *txr, uint32 mask, bool scrambled)
{
	uint32 size;
	pvr_ptr_t txraddr;
	void *data=NULL;

	size=fs_load(fn,&data);
	if(size==-1)
	{
		dbglog(DBG_ERROR, "LoadVqTxr: can't open file '%s'\n", fn);
		return(-1);
	}
	txraddr=pvr_mem_malloc(size);
	if(txraddr==NULL)
	{
		dbglog(DBG_ERROR, "LoadVqTxr: Unable to alocate pvr memory for '%s' (%d)\n", fn,size);
		return(-1);
	}
	memcpy2(txraddr,data,size);
	glGenTextures(1, txr);
	glBindTexture(GL_TEXTURE_2D, txr[0]);
	if(mask==PNG_MASK_ALPHA)
		glKosTex2D(GL_ARGB1555_TWID|GL_VQ_ENABLE, img_size, img_size,txraddr);
	else if(mask==PNG_FULL_ALPHA)
		glKosTex2D(GL_ARGB4444_TWID|GL_VQ_ENABLE, img_size, img_size,txraddr);
	else
		glKosTex2D(GL_RGB565_TWID|GL_VQ_ENABLE, img_size, img_size, txraddr);
	return(0);
}

void CGLDraw::LoadPngTxrToRAM(char* fn, uint32 mask, kos_img_t *img)
{
	void *tmpData;
	if(png_to_img(fn,mask,img))
	{
		printf("can't load %s\n",fn);
		return;
	}
	tmpData = memalign(32, img->w*img->h*2);
	memcpy(tmpData,img->data,img->w*img->h*2);
	kos_img_free(img,0);
	img->data=tmpData;
}
	
void CGLDraw::LoadTxrToVRAM(void *src, GLuint *txr, int lock, uint32 mask)
{
	pvr_ptr_t txraddr(NULL);
	txraddr=pvr_mem_malloc(512*512*2);
	pvr_txr_load_dma(src,txraddr,512*512*2,lock);
	glGenTextures(1,txr);
	glBindTexture(GL_TEXTURE_2D, *txr);
	if(mask==PNG_MASK_ALPHA)
		glKosTex2D(GL_ARGB1555,512,512,txraddr);
	else if(mask==PNG_FULL_ALPHA)
		glKosTex2D(GL_ARGB4444,512,512,txraddr);	
	else
		glKosTex2D(GL_RGB565,512,512, txraddr);
}

// Load a PNG texture to VRAM
void CGLDraw::LoadPngTxrToVRAM(const char *fn, GLuint *txr, uint32 mask, bool scramble)
{
	kos_img_t img;
	pvr_ptr_t txraddr;
	if(png_to_img(fn,mask,&img))
	{
		printf("can't load %s\n",fn);
		return;
	}
	txraddr=pvr_mem_malloc(img.w * img.h*2);
	pvr_txr_load_kimg(&img,txraddr,0);
	kos_img_free(&img,0);
	glGenTextures(1,txr);
	glBindTexture(GL_TEXTURE_2D, *txr);
	if(mask==PNG_MASK_ALPHA)
		glKosTex2D(GL_ARGB1555_TWID,img.w,img.h,txraddr);
	else if(mask==PNG_FULL_ALPHA)
		glKosTex2D(GL_ARGB4444_TWID,img.w,img.h,txraddr);	
	else
		glKosTex2D(GL_RGB565_TWID, img.w, img.h, txraddr);
}

// Load a PCX texture to VRAM
void CGLDraw::LoadPcxTxrToVRAM(const char *fn, GLuint *txr, bool scramble)
{
	kos_img_t img;
	pvr_ptr_t txraddr;

	if (pcx_to_img(fn, &img) < 0)
	{
		printf("can't load %s\n", fn);
		return;
	}

	txraddr = pvr_mem_malloc(img.w * img.h * 2);
	pvr_txr_load_kimg(&img, txraddr, PVR_TXRLOAD_INVERT_Y);
	kos_img_free(&img, 0);

	glGenTextures(1, txr);
	glBindTexture(GL_TEXTURE_2D, *txr);
	glKosTex2D(GL_RGB565_TWID, img.w, img.h, txraddr);
}

// Save the current OpenGL matrices
void CGLDraw::PushMatrices()
{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
}

// Get last saved OpenGL matrices
void CGLDraw::PopMatrices()
{
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}

// 2D mode
void CGLDraw::SetOrtho()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0,640,0,480,-1, 100);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

// 3D mode
void CGLDraw::SetPerspective()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f,640.0f/480.0f,0.1f,100.0f);
	glMatrixMode(GL_MODELVIEW);
}

void CGLDraw::SetScissor()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f,320.0f/240.0f,0.1f,100.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}
